import re
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.utils.translation import gettext_lazy as _


PAGE_SLUG_RE = r'[-\w/\.~]+'

def validate_url_chars(value):
    if re.match(PAGE_SLUG_RE, value) is None:
        raise ValidationError(_(u"Enter a valid 'slug' consisting of valid url characters. "
            u"These include uppercase and lowercase letters, decimal digits, "
            u"hyphen, period, underscore, and tilde."), code= u'invalid')


def validate_leading_slash(value):
    if value.startswith('/'):
        raise ValidationError(_(u"Cannot start with a slash"), code=u'invalid')


def validate_trailing_slash(value):
    if value.endswith('/'):
        raise ValidationError(_(u"Cannot end with a slash"), code=u'invalid')



from selectable.base import ModelLookup
from selectable.registry import registry
from taggit.models import Tag


class TagLookup(ModelLookup):
    model = Tag
    search_fields = ('name__icontains',)


registry.register(TagLookup)



from django.contrib.sitemaps import GenericSitemap

from pagelets.models import Page


class PageSiteMap(GenericSitemap):
    def __init__(self, priority=None, changefreq=None):
        info_dict = {
            'queryset': Page.objects.all(),
            'date_field': 'last_changed'
        }
        super(PageSiteMap, self).__init__(info_dict, priority, changefreq)



from django.conf import settings


CONTENT_AREAS = getattr(settings, 'PAGELET_CONTENT_AREAS', (
    ('main', 'Main'),
))
CONTENT_AREA_DEFAULT = getattr(settings, 'PAGELET_CONTENT_AREA_DEFAULT', 'main')

CONTENT_TYPES = getattr(settings, 'PAGELET_CONTENT_TYPES', (
    ('html', 'HTML',
     (),
     {},),
    ('markdown', 'Markdown',
     (),
     {},),
    ('wymeditor', 'WYMeditor',
     ('wymeditor/jquery.wymeditor.js',),
     {},),
    ('textile', 'Textile',
     (),
     {},),
)) + getattr(settings, 'PAGELET_CONTENT_TYPES_EXTRA', ())
CONTENT_TYPE_CHOICES = tuple((c[0], c[1]) for c in CONTENT_TYPES)
CONTENT_TYPE_DEFAULT = getattr(settings, 'PAGELET_CONTENT_TYPE_DEFAULT', 'html')

try:
    ATTACHMENT_PATH = settings.PAGELET_ATTACHMENT_PATH
except AttributeError:
    ATTACHMENT_PATH = getattr(settings, 'PAGE_ATTACHMENT_PATH', 'attachments/pages/')

# settings.PAGELET_TEMPLATE_TAGS is a list of template tag names that
# will load before each pagelet is rendered, allowing custom template
# tags to be included without including {% load <template_tag> %}
tags = set(['pagelet_tags'])
if hasattr(settings, 'PAGELET_TEMPLATE_TAGS'):
    for tag in settings.PAGELET_TEMPLATE_TAGS:
        tags.add(tag)
AUTO_LOAD_TEMPLATE_TAGS = '{%% load %s %%}' % ' '.join(tags)

BASE_TEMPLATES = getattr(settings, 'PAGELET_BASE_TEMPLATES', [])
BASE_TEMPLATE_DEFAULT = getattr(settings, 'PAGELET_BASE_TEMPLATE_DEFAULT', None)
INLINE_PAGELET_EXTRA = getattr(settings, 'PAGELET_INLINE_PAGELET_EXTRA', 0)
INLINE_SHARED_EXTRA = getattr(settings, 'PAGELET_INLINE_SHARED_EXTRA', 0)
INLINE_ATTACHMENT_EXTRA = getattr(settings, 'PAGELET_INLINE_ATTACHMENT_EXTRA', 0)



try:
    # Django 1.4+
    from django.conf.urls import patterns, url
except ImportError:
    # Django 1.3
    from django.conf.urls.defaults import patterns, url

from pagelets import views
from pagelets.validators import PAGE_SLUG_RE


urlpatterns = patterns('',
    url(
        r'^pagelet/(?:(?P<pagelet_slug>[^/]+)/)?create/$',
        views.create_pagelet,
        name='create_pagelet',
    ),
    url(
        r'^pagelet/(?P<pagelet_id>\d+)/edit/$',
        views.edit_pagelet,
        name='edit_pagelet',
    ),
    url(
        r'^pagelet/(?P<pagelet_id>\d+)/remove/$',
        views.remove_pagelet,
        name='remove_pagelet',
    ),
    url(
        r'^page/(?P<page_slug>%s)/attachment/upload/$' % PAGE_SLUG_RE,
        views.add_attachment,
        name='add_attachment',
    ),
    url(
        r'^page/(?P<page_slug>%s)/attachment/(?P<attachment_id>\d+)/remove/$' % PAGE_SLUG_RE,
        views.remove_attachment,
        name='remove_attachment',
    ),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import pagelets.validators
import taggit.managers


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('taggit', '__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, serialize=False, primary_key=True)),
                ('creation_date', models.DateTimeField(verbose_name='creation date', auto_now_add=True)),
                ('last_changed', models.DateTimeField(verbose_name='last changed', auto_now=True)),
                ('title', models.CharField(verbose_name='title', help_text="The page title.  To be displayed in the browser's title bar as well as at the top of the page.", max_length=255)),
                ('slug', models.CharField(unique=True, verbose_name='slug', max_length=255, help_text="A short string that uniquely identifies this page. It's used in the page URL, so don't change it unless you're positive nothing links to this page. Valid url chars include uppercase and lowercase letters, decimal digits, hyphen, period, underscore, and tilde. Do not include leading or trailing slashes.", validators=[pagelets.validators.validate_url_chars, pagelets.validators.validate_leading_slash, pagelets.validators.validate_trailing_slash])),
                ('description', models.TextField(blank=True, verbose_name='description', help_text='A description of the page for use in the meta tags and teaser or other short excepts')),
                ('base_template', models.CharField(blank=True, default='pagelets/view_page.html', verbose_name='base template', help_text='Specify an alternative layout template to use for this page.  Clear the selection to use the default layout.', max_length=255)),
                ('meta_keywords', models.CharField(blank=True, verbose_name='meta keywords', help_text='A comma delineated list of keywords', max_length=200)),
                ('meta_robots', models.CharField(blank=True, verbose_name='meta Robots', choices=[('FOLLOW, INDEX', 'FOLLOW, INDEX'), ('NOFOLLOW, NOINDEX', 'NOFOLLOW, NOINDEX'), ('FOLLOW, NOINDEX', 'FOLLOW, NOINDEX'), ('NOFOLLOW, INDEX', 'NOFOLLOW, INDEX')], max_length=20)),
                ('created_by', models.ForeignKey(editable=False, to=settings.AUTH_USER_MODEL, related_name='pagelets_page_created')),
                ('modified_by', models.ForeignKey(editable=False, to=settings.AUTH_USER_MODEL, related_name='pagelets_page_last_modified')),
                ('tags', taggit.managers.TaggableManager(to='taggit.Tag', verbose_name='Tags', through='taggit.TaggedItem', help_text='A comma-separated list of tags.')),
            ],
            options={
                'ordering': ('title',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageAttachment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=255)),
                ('file', models.FileField(upload_to='attachments/pages/')),
                ('order', models.SmallIntegerField(blank=True, choices=[(-10, -10), (-9, -9), (-8, -8), (-7, -7), (-6, -6), (-5, -5), (-4, -4), (-3, -3), (-2, -2), (-1, -1), (0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9), (10, 10)], null=True)),
                ('page', models.ForeignKey(related_name='attachments', to='pagelets.Page')),
            ],
            options={
                'ordering': ('order',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Pagelet',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, serialize=False, primary_key=True)),
                ('creation_date', models.DateTimeField(verbose_name='creation date', auto_now_add=True)),
                ('last_changed', models.DateTimeField(verbose_name='last changed', auto_now=True)),
                ('slug', models.CharField(blank=True, max_length=255, verbose_name='slug', help_text="A short string with no spaces or special characters that uniquely identifies this pagelet.  It may be used to link to load this pagelet dynamically from other places on the site, so don't change it unless you're positive nothing depends on the current name.", null=True)),
                ('css_classes', models.CharField(blank=True, verbose_name='CSS classes', help_text='Extra CSS classes, if any, to be added to the pagelet DIV in the HTML.', max_length=255)),
                ('type', models.CharField(default='html', verbose_name='content type', help_text="Controls the markup language and, in some cases, the JavaScript editor to be used for this pagelet's content.", max_length=32)),
                ('content', models.TextField(blank=True, verbose_name='content')),
            ],
            options={
                'ordering': ('slug',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='InlinePagelet',
            fields=[
                ('pagelet_ptr', models.OneToOneField(auto_created=True, serialize=False, to='pagelets.Pagelet', parent_link=True, primary_key=True)),
                ('area', models.CharField(default='main', verbose_name='content area', help_text='Specifies the placement of this pagelet on the page.', max_length=32)),
                ('order', models.SmallIntegerField(blank=True, choices=[(-10, -10), (-9, -9), (-8, -8), (-7, -7), (-6, -6), (-5, -5), (-4, -4), (-3, -3), (-2, -2), (-1, -1), (0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9), (10, 10)], help_text='The order in which pagelets should show up on the page. Lower numbers show up first.', null=True)),
                ('page', models.ForeignKey(related_name='inline_pagelets', to='pagelets.Page')),
            ],
            options={
                'ordering': ('order',),
            },
            bases=('pagelets.pagelet', models.Model),
        ),
        migrations.CreateModel(
            name='SharedPagelet',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, serialize=False, primary_key=True)),
                ('area', models.CharField(default='main', verbose_name='content area', help_text='Specifies the placement of this pagelet on the page.', max_length=32)),
                ('order', models.SmallIntegerField(blank=True, choices=[(-10, -10), (-9, -9), (-8, -8), (-7, -7), (-6, -6), (-5, -5), (-4, -4), (-3, -3), (-2, -2), (-1, -1), (0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9), (10, 10)], help_text='The order in which pagelets should show up on the page. Lower numbers show up first.', null=True)),
                ('page', models.ForeignKey(related_name='shared_pagelets', to='pagelets.Page')),
                ('pagelet', models.ForeignKey(to='pagelets.Pagelet')),
            ],
            options={
                'ordering': ('order',),
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='sharedpagelet',
            unique_together=set([('pagelet', 'page')]),
        ),
        migrations.AddField(
            model_name='pagelet',
            name='created_by',
            field=models.ForeignKey(editable=False, to=settings.AUTH_USER_MODEL, related_name='pagelets_pagelet_created'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='pagelet',
            name='modified_by',
            field=models.ForeignKey(editable=False, to=settings.AUTH_USER_MODEL, related_name='pagelets_pagelet_last_modified'),
            preserve_default=True,
        ),
    ]



import re

from django import template
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string
from django.template import RequestContext, Context

from pagelets.models import Pagelet, Page

register = template.Library()

try:
    basestring
except NameError:
    basestring = str


@register.inclusion_tag('pagelets/_render_pagelet.html', takes_context=True)
def render_pagelet(context, pagelet):
    """
    Renders the named pagelet in the calling template.
    """
    # don't modify the parent context
    parent_context = context
    if 'request' in context:
        context = RequestContext(parent_context['request'])
    else:
        context = Context()
    if isinstance(pagelet, basestring):
        # add the slug separately because we need it in the template even
        # if this pagelet doesn't exist
        context['pagelet_slug'] = pagelet
        try:
            pagelet = Pagelet.objects.get(slug=pagelet)
        except Pagelet.DoesNotExist:
            pagelet = None
    if pagelet:
        # add the slug separately because we need it in the template even
        # if this pagelet doesn't exist
        context['pagelet_slug'] = pagelet.slug
        context['page'] = parent_context.get('page', None)
        pagelet.rendered_content = pagelet.render(context)
    context['pagelet'] = pagelet
    context['include_links'] = True
    return context


@register.inclusion_tag('pagelets/_render_content_area.html',
                        takes_context=True)
def render_content_area(context, page, content_area):
    """
    Renders the named content area of the given page in the calling template.
    """
    # don't modify the parent context
    if 'request' in context:
        context = RequestContext(context['request'])
    else:
        context = Context()
    if isinstance(page, basestring):
        page = Page.objects.get(slug=page)
    context['page'] = page
    context['content_area'] = content_area
    context['pagelets'] = page.get_area_pagelets(content_area)
    return context


@register.inclusion_tag('pagelets/_pagelink_ifexists.html', takes_context=True)
def pagelink_ifexists(context, page, link_text):
    """
    Renders a link to the given page in the calling template.
    """
    # don't modify the parent context
    if 'request' in context:
        context = RequestContext(context['request'])
    else:
        context = Context()
    if isinstance(page, basestring):
        # add the slug separately because we need it in the template even
        # if this page doesn't exist
        context['page_slug'] = page
        try:
            page = Page.objects.get(slug=page)
        except Page.DoesNotExist:
            page = None

    if page:
        # add the slug separately because we need it in the template even
        # if this page doesn't exist
        context['page_slug'] = page.slug

    context['page'] = page
    context['link_text'] = link_text
    return context


@register.inclusion_tag('pagelets/_page_teaser.html', takes_context=True)
def page_content_teaser(context, page, num_words):
    """
    Renders a teaser of the given page object in the calling template.
    """
    # don't modify the parent context
    if 'request' in context:
        context = RequestContext(context['request'])
    else:
        context = Context()
    if isinstance(page, basestring):
        # add the slug separately because we need it in the template even
        # if this page doesn't exist
        context['page_slug'] = page
        try:
            page = Page.objects.get(slug=page)
        except Pagelet.DoesNotExist:
            page = None

    content = ''
    if page:
        # add the slug separately because we need it in the template even
        # if this page doesn't exist
        context['page_slug'] = page.slug
        for pagelet in page.pagelets.order_by('order'):
            pagelet.rendered_content = pagelet.render(context)
            content += render_to_string('pagelets/_render_pagelet.html',
                                        {'include_links': False,
                                         'pagelet': pagelet},
                                        context_instance=context)
    context['page'] = page
    context['content'] = content
    context['num_words'] = num_words
    return context


@register.inclusion_tag('pagelets/_page_teaser_new.html', takes_context=True)
def page_teaser(context, page, num_words):
    """
    Renders a better teaser of the given page object in the calling template.
    """
    # don't modify the parent context
    if 'request' in context:
        context = RequestContext(context['request'])
    else:
        context = Context()
    if isinstance(page, basestring):
        # add the slug separately because we need it in the template even
        # if this page doesn't exist
        context['page_slug'] = page
        try:
            page = Page.objects.get(slug=page)
        except Pagelet.DoesNotExist:
            page = None

    content = ''
    if page:
        # add the slug separately because we need it in the template even
        # if this page doesn't exist
        context['page_slug'] = page.slug
        if page.description:
            content = page.description
        else:
            pagelets = page.get_area_pagelets('main',
                                              with_shared=False)
            pagelets.sort(lambda a, b: len(b.content) - len(a.content))
            for pagelet in pagelets:
                search = re.findall('<p>(.*?)</p>', pagelet.render(context))
                if len(search) != 0:
                    content += ' '.join(search)
                else:
                    content += pagelet.render(context)

    context['page'] = page
    context['content'] = content
    context['num_words'] = num_words
    return context


@register.inclusion_tag('pagelets/_create_page.html', takes_context=True)
def create_page(context, link_text):
    """
    Renders a link to the admin to create a page based on the
    current request path. Meant to be used on a 404 page.
    """
    # don't modify the parent context
    if 'request' in context:
        request = context['request']
        context = RequestContext(request)
    else:
        return {'exists': True}

    has_perm = request.user.has_perm('pagelets.add_page')

    if has_perm:
        path = request.path.strip('/')
        try:
            exists = Page.objects.filter(slug=path).exists()
        except Page.DoesNotExist:
            exists = False
    else:
        path = ''
        exists = True

    admin_url = reverse('admin:%s_%s_add' % (Page._meta.app_label, Page._meta.module_name))

    context['exists'] = exists
    context['admin_url'] = u'{0}?slug={1}'.format(admin_url, path)
    context['link_text'] = link_text
    context['has_perm'] = has_perm
    return context



from django.db import models
from django.utils.translation import gettext_lazy as _
from django.utils.html import strip_tags
from django.core.urlresolvers import reverse
from django.conf import settings
from django.template import compile_string, TemplateSyntaxError, StringOrigin
from django.utils.encoding import python_2_unicode_compatible

from datetime import datetime

from pagelets import validators
from pagelets.utils import truncate_html_words
from pagelets import conf

from taggit.managers import TaggableManager


ORDER_CHOICES = [(x, x) for x in range(-10, 11)]


class PageletBase(models.Model):
    creation_date = models.DateTimeField(
        _('creation date'),
        auto_now_add=True,
        editable=False,
    )
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name='%(app_label)s_%(class)s_created',
        editable=False,
    )

    last_changed = models.DateTimeField(
        _('last changed'),
        auto_now=True,
        editable=False,
    )
    modified_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='%(app_label)s_%(class)s_last_modified',
        editable=False,
    )

    def save(self, **kwargs):
        if not self.id:
            self.creation_date = datetime.now()
        self.last_changed = datetime.now()
        super(PageletBase, self).save(**kwargs)

    class Meta:
        abstract = True


@python_2_unicode_compatible
class Page(PageletBase):
    title = models.CharField(
        _('title'),
        max_length=255,
        help_text=_('The page title.  To be displayed in the browser\'s title '
                    'bar as well as at the top of the page.')
    )
    slug = models.CharField(
        _('slug'),
        unique=True,
        max_length=255,
        help_text=_('A short string that uniquely identifies this page. '
                    'It\'s used in the page '
                    'URL, so don\'t change it unless you\'re positive nothing '
                    'links to this page. Valid url chars include '
                    'uppercase and lowercase letters, decimal digits, '
                    'hyphen, period, underscore, and tilde. '
                    'Do not include leading or trailing slashes.'),
        validators=[
            validators.validate_url_chars,
            validators.validate_leading_slash,
            validators.validate_trailing_slash
        ]
    )
    description = models.TextField(
        _('description'),
        blank=True,
        help_text=_('A description of the page for use in the meta tags and '
                    'teaser or other short excepts'),
    )
    base_template = models.CharField(
        _('base template'),
        max_length=255,
        blank=True,
        help_text=_('Specify an alternative layout template to use for this '
                    'page.  Clear the selection to use the default layout.'),
        default='pagelets/view_page.html',
    )
    meta_keywords = models.CharField(
        _('meta keywords'),
        max_length=200,
        help_text=_("A comma delineated list of keywords"),
        blank=True,
    )
    meta_robots = models.CharField(
        _('meta Robots'),
        max_length=20,
        blank=True,
        choices=[
            ('FOLLOW, INDEX', 'FOLLOW, INDEX'),
            ('NOFOLLOW, NOINDEX', 'NOFOLLOW, NOINDEX'),
            ('FOLLOW, NOINDEX', 'FOLLOW, NOINDEX'),
            ('NOFOLLOW, INDEX', 'NOFOLLOW, INDEX'),
        ]
    )

    tags = TaggableManager()

    def get_area_pagelets(self, area_slug, with_shared=True):
        """
        Combines and sorts the inline and shared pagelets for a given content
        area.  Pagelets without an order are given 0, so they show up
        in the middle.
        """
        pagelets = list(self.inline_pagelets.filter(area=area_slug))
        if with_shared:
            pagelets.extend(self.shared_pagelets.filter(area=area_slug))
        pagelets.sort(key=lambda a: a.order or 0)
        return pagelets

    def get_absolute_url(self):
        return reverse('view_page', kwargs={'page_slug': self.slug})

    class Meta:
        ordering = ('title',)

    def __str__(self):
        return self.title


@python_2_unicode_compatible
class Pagelet(PageletBase):
    """
    Primary model for storing pieces of static content in the database.
    """

    # whenever you need to reference a pagelet in CSS, use its slug
    slug = models.CharField(
        _('slug'),
        max_length=255,
        null=True,
        blank=True,
        help_text=_('A short string with no spaces or special characters that '
                    'uniquely identifies this pagelet.  It may be used to link '
                    'to load this pagelet dynamically from other places on the '
                    'site, so don\'t change it unless you\'re positive nothing '
                    'depends on the current name.'),
    )
    css_classes = models.CharField(
        _('CSS classes'),
        max_length=255,
        blank=True,
        help_text=_('Extra CSS classes, if any, to be added to the pagelet DIV '
                    'in the HTML.'),
    )
    type = models.CharField(
        _('content type'),
        max_length=32,
        default='html',
        help_text=_('Controls the markup language and, in some cases, the '
                    'JavaScript editor to be used for this pagelet\'s content.'),
    )
    content = models.TextField(_('content'), blank=True)

    # a property that consistently gives you access to the real "Pagelet"
    # instance for Pagelets, InlinePagelets, and SharedPagelets
    real = property(lambda self: self)

    def render(self, context):
        # pagelets can automagically use pagelets templatetags
        # in order to remove boilerplate
        loaded_cms = conf.AUTO_LOAD_TEMPLATE_TAGS + self.content
        """
        skip the first portions of render_to_string() ( finding the template )
         and go directly to compiling the template/pagelet
        render_to_string abbreviated:
                def render_to_string(template_name, dictionary=None, context_instance=None):
                       t = select/get_template(template_name)
                               template = get_template_from_string(source, origin, template_name)
                                       return Template(source, origin, name)
                       t.render(context_instance)

        """
        #XXX is this what the origin should be?
        origin = StringOrigin('pagelet: %s' % self.slug)
        compiled = compile_string(loaded_cms, origin).render(context)
        try:
            if self.type in ('html', 'tinymce', 'wymeditor'):
                html = compiled
            elif self.type == "textile":
                from textile import textile
                html = textile(str(compiled))
            elif self.type == "markdown":
                from markdown import markdown
                html = markdown(compiled)
            return html
        except TemplateSyntaxError as e:
            return 'Syntax error, %s' % e

        raise Exception("Unsupported template content type '%s'" % self.content.content_type)

    def save(self, *args, **kwargs):
        # force empty slugs to None so we don't get a DuplicateKey
        if self.slug == '':
            self.slug = None
        super(Pagelet, self).save(*args, **kwargs)

    class Meta:
        ordering = ('slug',)

    def __str__(self):
        if self.slug:
            return self.slug
        else:
            beginning = strip_tags(truncate_html_words(self.content, 5))
            if beginning:
                return beginning
            else:
                img_count = self.content.lower().count('<img')
                if img_count:
                    return "%s #%d (%s Images, no text)" % (type(self).__name__, self.id, img_count,)
                else:
                    return "%s #%d" % (type(self).__name__, self.id,)


class PlacedPageletBase(models.Model):
    """
    Abstract base model with the common fields for the inline and shared
    pagelet models.
    """
    area = models.CharField(
        _('content area'),
        max_length=32,
        default='main',
        help_text=_('Specifies the placement of this pagelet on the page.'),
    )
    order = models.SmallIntegerField(
        null=True,
        blank=True,
        choices=ORDER_CHOICES,
        help_text=_('The order in which pagelets should show up on the page. '
                    'Lower numbers show up first.'),
    )

    class Meta:
        abstract = True


class InlinePagelet(Pagelet, PlacedPageletBase):
    """
    A pagelet that shoes up on a single page.
    """
    page = models.ForeignKey(Page, related_name='inline_pagelets')

    # a property that consistently gives you access to the real "Pagelet"
    # instance for Pagelets, InlinePagelets, and SharedPagelets
    real = property(lambda self: self)

    class Meta:
        ordering = ('order',)


class SharedPagelet(PlacedPageletBase):
    """
    A pagelet that may show up on multiple pages.
    """
    pagelet = models.ForeignKey(Pagelet)
    page = models.ForeignKey(Page, related_name='shared_pagelets')

    def __init__(self, *args, **kwargs):
        super(SharedPagelet, self).__init__(*args, **kwargs)
        self.__pagelet_dirty = False

    def _get_slug(self):
        return self.pagelet.slug
    def _set_slug(self, slug):
        self.__pagelet_dirty = True
        self.pagelet.slug = slug
    slug = property(_get_slug, _set_slug)

    def _get_css_classes(self):
        return self.pagelet.css_classes
    def _set_css_classes(self, css_classes):
        self.__pagelet_dirty = True
        self.pagelet.css_classes = css_classes
    css_classes = property(_get_css_classes, _set_css_classes)

    def _get_type(self):
        return self.pagelet.type
    def _set_type(self, type):
        self.__pagelet_dirty = True
        self.pagelet.type = type
    type = property(_get_type, _set_type)

    def _get_content(self):
        return self.pagelet.content
    def _set_content(self, content):
        self.__pagelet_dirty = True
        self.pagelet.content = content
    content = property(_get_content, _set_content)

    # a property that consistently gives you access to the real "Pagelet"
    # instance for Pagelets, InlinePagelets, and SharedPagelets
    real = property(lambda self: self.pagelet)

    def render(self, *args, **kwargs):
        return self.pagelet.render(*args, **kwargs)

    def save(self, *args, **kwargs):
        if self.__pagelet_dirty:
            self.pagelet.save()
            self.__pagelet_dirty = False
        return super(SharedPagelet, self).save(*args, **kwargs)

    class Meta:
        unique_together = (('pagelet', 'page'),)
        ordering = ('order',)


@python_2_unicode_compatible
class PageAttachment(models.Model):
    page = models.ForeignKey(Page, related_name='attachments')
    name = models.CharField(max_length=255)
    file = models.FileField(upload_to=conf.ATTACHMENT_PATH)
    order = models.SmallIntegerField(
        null=True,
        blank=True,
        choices=ORDER_CHOICES,
    )

    class Meta:
        ordering = ('order',)

    def __str__(self):
        return self.name



try:
    # Django 1.4+
    from django.conf.urls import include, patterns, url
except ImportError:
    # Django 1.3
    from django.conf.urls.defaults import include, patterns, url

urlpatterns = patterns('',
    url(r'^management/', include('pagelets.urls.management')),
    url(r'^', include('pagelets.urls.content')),
)



try:
    # Django 1.4+
    from django.conf.urls import include, patterns, url
except ImportError:
    # Django 1.3
    from django.conf.urls.defaults import include, patterns, url
from django.contrib import admin


admin.autodiscover()

urlpatterns = patterns('',
    url(r'^pagelets/', include('pagelets.urls')),
    url(r'^admin/', include(admin.site.urls)),
)



import hashlib

from django.conf import settings
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.shortcuts import get_object_or_404
from django.views.decorators.http import condition
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.decorators import user_passes_test
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.db.models import Max

from pagelets.models import Pagelet, InlinePagelet, Page, PageAttachment
from pagelets.forms import PageletForm, UploadForm
from pagelets import conf


def view_page(request, page_slug, template='pagelets/view_page.html'):
    page = get_object_or_404(Page, slug=page_slug)

    context = {
        'page': page,
    }
    return render_to_response(
        page.base_template or template,
        context,
        context_instance=RequestContext(request),
    )


@user_passes_test(lambda u: u.has_perm('pagelets.add_pagelet'),
                  login_url=settings.LOGIN_URL)
def create_pagelet(request, pagelet_slug=None):
    page = None
    if 'page_id' in request.GET:
        try:
            page_id = int(request.GET['page_id'])
            page = Page.objects.get(pk=page_id)
        except (Page.DoesNotExist, ValueError):
            pass
    content_area = ''
    if 'content_area' in request.GET and\
       request.GET['content_area'] in [slug for slug, name in conf.CONTENT_AREAS]:
        content_area = request.GET['content_area']
    pagelet = None
    if pagelet_slug:
        try:
            pagelet = Pagelet.objects.get(slug=pagelet_slug)
        except Pagelet.DoesNotExist:
            pass
    if not pagelet:
        order = None
        if page:
            # if the page exists, set the order of this pagelet to
            # max(page.inline_pagelets.order) + 1
            order = page.inline_pagelets.aggregate(Max('order'))['order__max'] or 0
            order += 1
            pagelet = InlinePagelet.objects.create(
                slug=pagelet_slug or '',
                created_by=request.user,
                modified_by=request.user,
                page=page,
                area=content_area,
                order=order,
            )
        else:
            pagelet = Pagelet.objects.create(
                slug=pagelet_slug or '',
                created_by=request.user,
                modified_by=request.user,
            )
    edit_pagelet = reverse('edit_pagelet', kwargs={'pagelet_id': pagelet.id})
    if 'next' in request.GET:
        edit_pagelet += '?next=%s' % request.GET['next']
    return HttpResponseRedirect(edit_pagelet)


def _last_modified(request, pagelet_id):
    pagelet = get_object_or_404(Pagelet, pk=pagelet_id)
    return pagelet.last_changed


def _etag(request, pagelet_id):
    pagelet = get_object_or_404(Pagelet, pk=pagelet_id)
    etag = hashlib.md5(str(sorted(vars(pagelet).items())).encode('utf8')).hexdigest()
    return etag


@condition(last_modified_func=_last_modified, etag_func=_etag)
def edit_pagelet(
    request,
    pagelet_id,
    template='pagelets/edit_pagelet.html',
    redirect_field_name=REDIRECT_FIELD_NAME,
    redirect_to=None,
):
    redirect_to = request.REQUEST.get(redirect_field_name, redirect_to)
    if not redirect_to:
        redirect_to = '/'

    pagelet = get_object_or_404(Pagelet, pk=pagelet_id)

    @user_passes_test(
        lambda u: u.has_perm('pagelets.change_pagelet'),
        login_url=settings.LOGIN_URL)
    def _(request):
        preview_form = None
        pagelet_preview = None
        if request.POST:
            form = PageletForm(request.POST, instance=pagelet)
            if form.is_valid():
                if 'save_btn' in request.REQUEST:
                    form.save(user=request.user)
                    return HttpResponseRedirect(redirect_to)
                else:
                    preview_form = PageletForm(
                        request.POST,
                        instance=pagelet,
                        preview=True,
                    )
                    pagelet_preview = form.save(commit=False, user=request.user)
        else:
            form = PageletForm(instance=pagelet)

        context = {
            'form': form,
            'pagelet': pagelet,
            'preview_form': preview_form,
            'pagelet_preview': pagelet_preview,
        }

        return render_to_response(
            template,
            context,
            context_instance=RequestContext(request),
        )
    return _(request)


@user_passes_test(lambda u: u.has_perm('pagelets.delete_pagelet'),
                  login_url=settings.LOGIN_URL)
def remove_pagelet(
    request,
    pagelet_id,
    template='pagelets/remove_pagelet.html',
    redirect_field_name=REDIRECT_FIELD_NAME,
    redirect_to=None,
):
    pagelet = get_object_or_404(Pagelet, pk=pagelet_id)

    redirect_to = request.REQUEST.get(redirect_field_name, redirect_to)
    if not redirect_to:
        redirect_to = '/'

    if request.method == 'POST':
        pagelet.delete()
        messages.info(request, 'Pagelet successfully deleted.')
        return HttpResponseRedirect(redirect_to)
    return render_to_response(
        template,
        {'pagelet': pagelet},
        context_instance=RequestContext(request),
    )


@user_passes_test(lambda u: u.has_perm('pagelets.add_pageattachment'),
                  login_url=settings.LOGIN_URL)
def add_attachment(
        request,
        page_slug,
        template='pagelets/attach.html',
):
    page = get_object_or_404(Page, slug=page_slug)
    if request.POST:
        form = UploadForm(request.POST, request.FILES)
        if form.is_valid():
            form.save(page=page)
            return HttpResponseRedirect(
                reverse('view_page', args=[page.slug])
            )

    else:
        form = UploadForm()
    context = {
        'page': page,
        'form': form,
    }
    return render_to_response(
        template,
        context,
        context_instance=RequestContext(request),
    )


@user_passes_test(lambda u: u.has_perm('pagelets.delete_pageattachment'),
                  login_url=settings.LOGIN_URL)
def remove_attachment(request, page_slug, attachment_id):
    attachment = get_object_or_404(
        PageAttachment,
        pk=attachment_id,
        page__slug=page_slug,
    )
    attachment.delete()
    if 'next' in request.GET:
        return HttpResponseRedirect(request.GET['next'])
    else:
        return HttpResponseRedirect('/')



from django.conf import settings
from django import forms
from django.utils.translation import gettext_lazy as _

import selectable.forms

from pagelets.models import Page, PageletBase, Pagelet, InlinePagelet, SharedPagelet, PageAttachment
from pagelets.lookups import TagLookup
from pagelets import conf


def get_pagelet_type_assets(pagelet_type=None, base_scripts=None, base_styles=None):
    all_scripts = [] if base_scripts is None else list(base_scripts)
    all_styles = {} if base_styles is None else base_styles
    for k, v in all_styles.items():
        all_styles[k] = list(v)

    for (val, label, scripts, styles) in conf.CONTENT_TYPES:
        if pagelet_type is None or val == pagelet_type:
            for script in scripts:
                if script not in all_scripts:
                    all_scripts.append(script)
            for key in styles:
                all_styles.setdefault(key, [])
                for stylesheet in styles:
                    if stylesheet not in all_styles[key]:
                        all_styles[key].append(stylesheet)
    return tuple(all_scripts), dict((k, tuple(v)) for (k, v) in all_styles.items())


class BasePageletForm(forms.ModelForm):

    class Meta:
        model = PageletBase
        fields = ()
        widgets = {
            "type": forms.Select(choices=conf.CONTENT_TYPE_CHOICES),
            "area": forms.Select(choices=conf.CONTENT_AREAS)
        }

    class Media:
        css = {
            'all': ('css/pagelets.css',)
        }
        js = ('js/pagelets.js',)

        js, css = get_pagelet_type_assets(base_scripts=js, base_styles=css)

    def __init__(self, *args, **kwargs):
        initial = kwargs.setdefault('initial', {})
        initial.setdefault('type', conf.CONTENT_TYPE_DEFAULT)
        initial.setdefault('area', conf.CONTENT_AREA_DEFAULT)
        super(BasePageletForm, self).__init__(*args, **kwargs)

    def save(self, commit=True, user=None):
        instance = super(BasePageletForm, self).save(commit=False)
        if user:
            instance.created_by = user
            instance.modified_by = user
        elif commit:
            raise ValueError(_(u'A user is required when saving a Pagelet'))
        if commit:
            instance.save()
        return instance


class ContentPageletForm(BasePageletForm):
    def __init__(self, *args, **kwargs):
        self.preview = kwargs.pop('preview', False)
        super(BasePageletForm, self).__init__(*args, **kwargs)
        if self.preview:
            for field in self.fields.values():
                field.widget = forms.HiddenInput()
        else:
            self.fields['content'].widget = forms.Textarea(
                attrs={'rows': 30, 'cols': 90}
            )


class InlinePageletForm(ContentPageletForm):
    class Meta:
        model = InlinePagelet
        fields = ('type', 'content', 'area')
        widgets = {
            "type": forms.Select(choices=conf.CONTENT_TYPE_CHOICES),
            "area": forms.Select(choices=conf.CONTENT_AREAS)
        }


class SharedPageletForm(BasePageletForm):
    class Meta:
        model = SharedPagelet
        fields = ('pagelet', 'area', 'order')
        widgets = {
            "area": forms.Select(choices=conf.CONTENT_AREAS),
        }


class PageletForm(ContentPageletForm):
    class Meta:
        model = Pagelet
        fields = ('type', 'content')


class UploadForm(forms.ModelForm):

    class Meta:
        model = PageAttachment
        fields = ('name', 'file', 'order')

    def save(self, page, commit=True):
        instance = super(UploadForm, self).save(commit=False)
        instance.page = page
        if commit:
            instance.save()
        return instance



class PageForm(forms.ModelForm):
    class Meta:
        model = Page
        fields = ('title', 'slug', 'tags', 'description', 'meta_keywords', 'meta_robots', 'base_template')

    tags = selectable.forms.AutoCompleteSelectMultipleField(
        lookup_class=TagLookup,
        label='Select a tag',
        required=False,
    )
    base_template = forms.CharField(widget=forms.Select(choices=conf.BASE_TEMPLATES),
        initial=conf.BASE_TEMPLATE_DEFAULT)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.instance and self.instance.pk:
            self.initial['tags'] = self.instance.tags.all().values_list('pk', flat=True)

    def save(self, *args, **kwargs):
        ret = super().save(*args, **kwargs)
        ret._pending_tags = set(tag.name for tag in self.cleaned_data['tags'])
        return ret



from django.test import TestCase, Client
from django.template import compile_string, StringOrigin
from django.template.context import Context
from django.contrib.auth.models import User, Permission
from django.core.urlresolvers import reverse
from django.core.exceptions import ValidationError

from pagelets.models import Page, Pagelet


class PageletsTest(TestCase):
    def setUp(self):
        self.editor = User.objects.create_user(
            username='editor',
            email='editor@example.com',
            password='abc123',
        )
        change_pagelet = Permission.objects.get(
            codename='change_pagelet',
            content_type__app_label='pagelets',
            content_type__model='pagelet',
        )
        add_pagelet = Permission.objects.get(
            codename='add_pagelet',
            content_type__app_label='pagelets',
            content_type__model='pagelet',
        )
        self.editor.user_permissions.add(change_pagelet)
        self.editor.user_permissions.add(add_pagelet)
        self.c = Client()

    def testViewNonexistentPagelet(self):
        template_str = """{% spaceless %}
{% load pagelet_tags %}
{% render_pagelet 'nonexistent-pagelet' %}
{% endspaceless %}"""
        origin = StringOrigin('test')
        compiled = compile_string(template_str, origin).render(Context())
        self.assertEqual(compiled, '<div class="pagelet nonexistent-pagelet"><div class="pagelet-content"></div></div>')

    def testCreateNotexistentPagelet(self):
        self.c.login(username=self.editor.username, password='abc123')
        url = reverse('create_pagelet', kwargs={'pagelet_slug': 'new-pagelet'})
        response = self.c.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Pagelet.objects.count(), 1)
        pagelet = Pagelet.objects.get()
        url = reverse('edit_pagelet', kwargs={'pagelet_id': pagelet.id})
        self.assertRedirects(response, url)

    def testEditPagelet(self):
        self.c.login(username=self.editor.username, password='abc123')
        pagelet = Pagelet.objects.create(
            created_by=self.editor,
            modified_by=self.editor,
        )
        url = reverse('edit_pagelet', kwargs={'pagelet_id': pagelet.real.pk})
        response = self.c.get(url, follow=True)
        self.assertEqual(response.status_code, 200, response)
        data = {
            'type': 'html',
            'content': '<p>new content</p>',
        }
        response = self.c.post(url, data)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, data['content'])
        self.assertContains(response, 'Preview pagelet #%s' % pagelet.id)
        data['save_btn'] = 'Save'
        response = self.c.post(url + '?next=/', data)
        self.assertEqual(302, response.status_code)
        pagelet = Pagelet.objects.get(pk=pagelet.id)
        self.assertEqual(pagelet.content, data['content'])

    def testViewPage(self):
        page = Page.objects.create(
            title='New Page',
            slug='new-page',
            created_by=self.editor,
            modified_by=self.editor,
        )
        pagelet = page.inline_pagelets.create(
            content='<p>main content</p>',
            css_classes='main',
            created_by=self.editor,
            modified_by=self.editor,
        )
        url = reverse('view_page', kwargs={'page_slug': page.slug})
        response = self.c.get(url)
        self.assertContains(response, pagelet.content)


class PageTestCase(TestCase):
    def setUp(self):
        self.editor = User.objects.create_user(
            username='editor',
            email='editor@example.com',
            password='abc123',
        )
        change_pagelet = Permission.objects.get(
            codename='change_pagelet',
            content_type__app_label='pagelets',
            content_type__model='pagelet',
        )
        add_pagelet = Permission.objects.get(
            codename='add_pagelet',
            content_type__app_label='pagelets',
            content_type__model='pagelet',
        )
        self.editor.user_permissions.add(change_pagelet)
        self.editor.user_permissions.add(add_pagelet)

    def testNoLeadingSlash(self):
        "Don't allow a leading slash in the page slug."
        test_page = Page(
            title='New Page',
            slug='/new-page',
            created_by=self.editor,
            modified_by=self.editor,
        )
        self.assertRaises(ValidationError, test_page.full_clean)

    def testNoTrailingSlash(self):
        "Don't allow a trailing slash in the page slug."
        test_page = Page(
            title='New Page',
            slug='new-page/',
            created_by=self.editor,
            modified_by=self.editor,
        )
        self.assertRaises(ValidationError, test_page.full_clean)

    def testRFC3986Characters(self):
        """
        Page slug should only contain characters defined in
        RFC 3986: http://www.ietf.org/rfc/rfc3986.txt
        along with additional slashes.
        """
        test_page = Page(
            title='New Page',
            slug='a/A/1/./~/-/_',
            created_by=self.editor,
            modified_by=self.editor,
        )
        try:
            test_page.full_clean()
        except ValidationError as e:
            self.fail(str(e))



from setuptools import setup, find_packages

setup(
    name='django-pagelets',
    version=__import__('pagelets').__version__,
    author='Caktus Consulting Group',
    author_email='solutions@caktusgroup.com',
    packages=find_packages(exclude=['sample_project']),
    include_package_data=True,
    url='http://github.com/caktus/django-pagelets/',
    license='BSD',
    description='Simple, flexible app for integrating static, unstructured '
                'content in a Django site',
    classifiers=[
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
    ],
    long_description=open('README.rst').read(),
    zip_safe=False, # because we're including media that Django needs
)



try:
    # Django 1.4+
    from django.conf.urls import patterns, url
except ImportError:
    # Django 1.3
    from django.conf.urls.defaults import patterns, url

from pagelets import views
from pagelets.validators import PAGE_SLUG_RE


urlpatterns = patterns('',
    url(
        r'^(?P<page_slug>%s)/$' % PAGE_SLUG_RE,
        views.view_page,
        name='view_page',
    ),
)



from django.contrib import admin
from django.conf import settings
from django.utils.html import strip_tags
from django import forms

from pagelets.forms import PageForm, InlinePageletForm, SharedPageletForm, PageletForm
from pagelets.utils import truncate_html_words
from pagelets import models as pagelets
from pagelets import conf

if 'treenav' in settings.INSTALLED_APPS:
    from treenav.admin import GenericMenuItemInline
else:
    GenericMenuItemInline = None


JS_URLS = [
    'wymeditor/jquery.wymeditor.js',
    'js/pagelets.js',
]
for code, name, js, css in conf.CONTENT_TYPES:
    JS_URLS.extend(js)


class InlinePageletAdmin(admin.StackedInline):
    model = pagelets.InlinePagelet
    extra = conf.INLINE_PAGELET_EXTRA
    fk_name = 'page'
    form = InlinePageletForm
    fieldsets = (
        (None, {
            'fields': ('type', 'content')
        }),
        ('Style and Placement', {
            'classes': ('collapse',),
            'fields': ('css_classes', ('area', 'order')),
        }),
    )


class SharedPageletAdmin(admin.StackedInline):
    model = pagelets.SharedPagelet
    extra = conf.INLINE_SHARED_EXTRA
    fk_name = 'page'
    form = SharedPageletForm
    fieldsets = (
        (None, {
            'fields': ('pagelet', ('area', 'order'))
        }),
    )


class InlinePageAttachmentAdmin(admin.StackedInline):
    model = pagelets.PageAttachment
    extra = conf.INLINE_ATTACHMENT_EXTRA
    fieldsets = (
        (None, {
            'fields': (('name', 'order'), 'file')
        }),
    )


class PageAdmin(admin.ModelAdmin):
    list_display = (
        'title',
        'slug',
        'modified_by',
        'last_changed',
        'created_by',
        'creation_date',
    )
    form = PageForm
    search_fields = ('title',)
    list_filter = ('modified_by',)
    prepopulated_fields = {'slug': ('title',)}
    inlines = [InlinePageletAdmin, SharedPageletAdmin,
               InlinePageAttachmentAdmin]
    shown_fields = ['title', 'slug', 'tags']
    optional_fields = ['description', ('meta_keywords', 'meta_robots')]
    if getattr(settings, 'PAGELET_BASE_TEMPLATES', None):
        optional_fields.insert(0, 'base_template')
    fieldsets = (
        (None, {
            'fields': shown_fields,
        }),
        ('Optional Information', {
            'classes': ('collapse',),
            'fields': optional_fields,
        }),
    )
    if GenericMenuItemInline:
        inlines.insert(0, GenericMenuItemInline)

    class Media:
        css = {
            'all': ('css/wymeditor.admin.css',)
        }
        js = JS_URLS

    def save_model(self, request, obj, form, change):
        if not obj.id:
            obj.created_by = request.user
            obj.modified_by = request.user
        obj.save()
        if hasattr(obj, '_pending_tags'):
            obj.tags.set(*obj._pending_tags)

    def save_formset(self, request, form, formset, change):
        pagelets = formset.save(commit=False)
        deleted = [f.instance for f in formset.deleted_forms]
        for pagelet in deleted:
            pagelet.delete()
        for pagelet in pagelets:
            pagelet.created_by = request.user
            pagelet.modified_by = request.user
            pagelet.save()
        formset.save_m2m()
admin.site.register(pagelets.Page, PageAdmin)


class PageletAdmin(admin.ModelAdmin):
    list_display = (
        'id',
        'slug',
        'type',
        'modified_by',
        'last_changed',
        'created_by',
        'creation_date',
        'content_preview',
    )
    search_fields = ('slug', 'content',)
    list_filter = ('type', 'modified_by', 'last_changed', 'creation_date')
    form = PageletForm

    class Media:
        css = {
            'all': ('css/wymeditor.admin.css',)
        }
        js = JS_URLS

    def content_preview(self, obj):
        return strip_tags(truncate_html_words(obj.content, 5))
    content_preview.short_description = 'content preview'

    def save_model(self, request, obj, form, change):
        if not obj.id:
            obj.created_by = request.user
            obj.modified_by = request.user
        obj.save()
admin.site.register(pagelets.Pagelet, PageletAdmin)




def truncate_html_words(value, num_words):
    """Backwords compatibility shim."""
    try:
        # Django 1.4+
        from django.utils.text import Truncator
    except ImportError:
        # Django 1.3
        from django.utils.text import truncate_html_words
        return truncate_html_words(value, num_words)
    else:
        return Truncator(value).words(num_words, html=True)
